package com.kdanmobile.admanager.admob

import android.app.Activity
import android.content.Context
import android.os.Build
import com.google.ads.consent.*
import com.kdanmobile.admanager.BuildConfig
import java.net.URL

class AdmobConsentManager(
        context: Context,
        private val publisherIds: Array<String>,
        private val privacyPolicyUrl: String) {

    private val context = context.applicationContext
    private val consentInformation = ConsentInformation.getInstance(this.context)

    init {
        if (BuildConfig.DEBUG) {
            consentInformation.addTestDevice("D5F152A9CC95D9A54DAD4106EF071119") // kdan pixel 1
            consentInformation.debugGeography = DebugGeography.DEBUG_GEOGRAPHY_EEA
        }
    }

    fun isUnknown(): Boolean {
        return consentInformation.consentStatus == ConsentStatus.UNKNOWN
    }

    fun isPersonalized(): Boolean {
        return consentInformation.consentStatus == ConsentStatus.PERSONALIZED
    }

    fun isInEEAOrUnknown(): Boolean {
        return consentInformation.isRequestLocationInEeaOrUnknown
    }

    fun canPersonalized(): Boolean {
        if (!isInEEAOrUnknown()) return true
        return isPersonalized()
    }

    @JvmOverloads fun requestInfoUpdate(onSuc: Runnable? = null, onFailed: Runnable? = null) {
        consentInformation.requestConsentInfoUpdate(publisherIds, object : ConsentInfoUpdateListener {
            override fun onFailedToUpdateConsentInfo(reason: String?) {
                onFailed?.run()
            }

            override fun onConsentInfoUpdated(consentStatus: ConsentStatus?) {
                onSuc?.run()
            }
        })
    }

    @JvmOverloads fun loadThenShowAsync(
            activity: Activity,
            listener: Listener = Listener()) {
        var form: ConsentForm? = null
        form = ConsentForm.Builder(activity, URL(privacyPolicyUrl))
                .withListener(object : ConsentFormListener() {
                    override fun onConsentFormOpened() {
                        super.onConsentFormOpened()
                        listener.onOpened()
                    }

                    override fun onConsentFormLoaded() {
                        super.onConsentFormLoaded()
                        listener.onLoaded()
                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1) {
                            if (activity.isDestroyed) return
                        }
                        if (activity.isFinishing) return
                        form?.show()
                    }

                    override fun onConsentFormError(reason: String?) {
                        super.onConsentFormError(reason)
                        listener.onError(reason)
                    }

                    override fun onConsentFormClosed(consentStatus: ConsentStatus?, userPrefersAdFree: Boolean?) {
                        super.onConsentFormClosed(consentStatus, userPrefersAdFree)
                        listener.onClosed(userPrefersAdFree ?: false)
                    }
                })
                .withPersonalizedAdsOption()
                .withNonPersonalizedAdsOption()
                .withAdFreeOption()
                .build()
        form.load()
    }

    open class Listener {
        open fun onOpened() {}
        open fun onLoaded() {}
        open fun onError(reason: String?) {}
        open fun onClosed(userPrefersAdFree: Boolean) {}
    }
}