package com.kdanmobile.admanager.admob

import android.app.Activity
import com.google.android.gms.ads.InterstitialAd
import com.kdanmobile.admanager.InterstitialAdListener
import com.kdanmobile.admanager.InterstitialAdManager
import java.util.concurrent.CopyOnWriteArrayList

class AdmobInterstitialAdManager : InterstitialAdManager {

    private val interstitialMap = HashMap<String, InterstitialAd>()
    private val adListeners = CopyOnWriteArrayList<InterstitialAdListener>()
    override var isPersonalized: Boolean = true

    override fun request(adUnitId: String) {
        interstitialMap[adUnitId]?.let {
            if (it.isLoading) return
            if (it.isLoaded) return
            it.loadAd(RequestFactory.create(isPersonalized))
        }
    }

    override fun request(activity: Activity, adUnitId: String) {
        val context = activity.application
        var interstitial = interstitialMap[adUnitId]
        if (interstitial == null) {
            interstitial = InterstitialAd(context).apply {
                this.adUnitId = adUnitId
                this.adListener = object : com.google.android.gms.ads.AdListener() {
                    override fun onAdOpened() {
                        super.onAdOpened()
                        adListeners.forEach { it.onImpressed(adUnitId) }
                    }

                    override fun onAdLeftApplication() {
                        super.onAdLeftApplication()
                        adListeners.forEach { it.onClicked(adUnitId) }
                    }

                    override fun onAdFailedToLoad(p0: Int) {
                        super.onAdFailedToLoad(p0)
                        adListeners.forEach { it.onFailedToLoad(adUnitId) }
                    }

                    override fun onAdClosed() {
                        super.onAdClosed()
                        adListeners.forEach { it.onClosed(adUnitId) }
                    }

                    override fun onAdLoaded() {
                        super.onAdLoaded()
                        adListeners.forEach { it.onLoaded(adUnitId) }
                    }
                }
            }
            interstitialMap[adUnitId] = interstitial
        }
        request(adUnitId)
    }

    override fun show(adUnitId: String): Boolean {
        interstitialMap[adUnitId]?.let {
            if (it.isLoaded) {
                it.show()
                return true
            }
        }
        return false
    }

    override fun destroy(activity: Activity) {
        // Do nothing, Admob does not need to destroy
    }

    override fun registerListener(interstitialAdListener: InterstitialAdListener) {
        adListeners.add(0, interstitialAdListener)
    }

    override fun unregisterListener(interstitialAdListener: InterstitialAdListener) {
        adListeners.remove(interstitialAdListener)
    }

    override fun isLoading(adUnitId: String): Boolean {
        interstitialMap[adUnitId]?.let {
            return it.isLoading
        }
        return false
    }

    override fun isLoaded(adUnitId: String): Boolean {
        interstitialMap[adUnitId]?.let {
            return it.isLoaded
        }
        return false
    }
}