package com.kdanmobile.admanager.mopub

import android.app.Activity
import android.content.Context
import android.view.View
import com.kdanmobile.admanager.*
import com.mopub.common.MoPub
import com.mopub.common.SdkConfiguration
import java.util.concurrent.CopyOnWriteArrayList

class MoPubAdManager : AdManager {
    private lateinit var context: Context
    private lateinit var interstitialAdManager: InterstitialAdManager
    private lateinit var nativeAdManager: NativeAdManager

    override var isInitialized = false
    override val onInitListeners = CopyOnWriteArrayList<Runnable>()
    override var isPersonalized: Boolean = true
        set(value) {
            field = value
            interstitialAdManager.isPersonalized = value
            nativeAdManager.isPersonalized = value
        }

    override fun init(context: Context, adAppId: String, adUnitId: String, volume: Float) {
        this.context = context

        val configuration = SdkConfiguration.Builder(adUnitId).build()
        MoPub.initializeSdk(context, configuration) {
            isInitialized = true
            onInitListeners.forEach {
                it.run()
            }
        }

        interstitialAdManager = MoPubInterstitialAdManager()
        nativeAdManager = MoPubNativeAdManager(context)

        onInitListeners.forEach { it.run() }
    }

    override fun requestInterstitial(adUnitId: String) {
        interstitialAdManager.request(adUnitId)
    }

    override fun requestInterstitial(activity: Activity, adUnitId: String) {
        interstitialAdManager.request(activity, adUnitId)
    }

    override fun showInterstitial(adUnitId: String): Boolean {
        return interstitialAdManager.show(adUnitId)
    }

    override fun destroyInterstitial(activity: Activity) {
        interstitialAdManager.destroy(activity)
    }

    override fun registerInterstitialAdListener(interstitialAdListener: InterstitialAdListener) {
        interstitialAdManager.registerListener(interstitialAdListener)
    }

    override fun unregisterInterstitialAdListener(interstitialAdListener: InterstitialAdListener) {
        interstitialAdManager.unregisterListener(interstitialAdListener)
    }

    override fun isInterstitialLoading(adUnitId: String): Boolean {
        return interstitialAdManager.isLoading(adUnitId)
    }

    override fun isInterstitialLoaded(adUnitId: String): Boolean {
        return interstitialAdManager.isLoaded(adUnitId)
    }

    override fun registerNativeAdListener(nativeAdListener: NativeAdListener) {
        nativeAdManager.listener += nativeAdListener
    }

    override fun unregisterNativeAdListener(nativeAdListener: NativeAdListener) {
        nativeAdManager.listener -= nativeAdListener
    }

    override fun requestNative(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder) {
        nativeAdManager.request(adUnitId, nativeAdViewBinder)
    }

    override fun takeNativeAdView(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder): View? {
        return nativeAdManager.takeAdView(adUnitId, nativeAdViewBinder)
    }

    override fun isNatvieLoading(adUnitId: String): Boolean {
        return nativeAdManager.isLoading(adUnitId)
    }
}