package com.kdanmobile.admanager.mopub

import android.content.Context
import android.view.View
import com.kdanmobile.admanager.NativeAdManager
import com.kdanmobile.admanager.NativeAdListener
import com.kdanmobile.admanager.NativeAdViewBinder
import com.mopub.common.MoPub
import com.mopub.nativeads.*
import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.CopyOnWriteArrayList

class MoPubNativeAdManager(private val context: Context) : NativeAdManager {

    override var isPersonalized: Boolean = true
        set(value) {
            field = value
            MoPub.getPersonalInformationManager()?.let {
                when (value) {
                    true -> it.grantConsent()
                    false -> it.revokeConsent()
                }
            }
        }
    override val listener = CopyOnWriteArrayList<NativeAdListener>()

    private val nativeAdMap = ConcurrentHashMap<String, NativeAd>()
    private val isLoadingMap = ConcurrentHashMap<String, Boolean>()

    override fun request(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder) {
        if (isLoading(adUnitId)) return

        val listener = object : MoPubNative.MoPubNativeNetworkListener {
            override fun onNativeLoad(nativeAd: NativeAd?) {
                if (nativeAd == null) return
                nativeAdMap[adUnitId] = nativeAd
                isLoadingMap[adUnitId] = false
                onLoaded(adUnitId)
            }

            override fun onNativeFail(errorCode: NativeErrorCode?) {
                isLoadingMap[adUnitId] = false
                onFailedToLoad(adUnitId)
            }
        }
        isLoadingMap[adUnitId] = true
        MoPubNative(context, adUnitId, listener).apply {
            ViewBinder.Builder(nativeAdViewBinder.layout).apply {
                mainImageId(nativeAdViewBinder.mainImageViewId)
                iconImageId(nativeAdViewBinder.iconImageViewId)
                titleId(nativeAdViewBinder.titleTextViewId)
                textId(nativeAdViewBinder.textTextViewId)
                callToActionId(nativeAdViewBinder.ctaButtonId)
                privacyInformationIconImageId(nativeAdViewBinder.privacyInformationIcon)
            }.build().let { registerAdRenderer(MoPubStaticNativeAdRenderer(it)) }

            MediaViewBinder.Builder(nativeAdViewBinder.layout).apply {
                mediaLayoutId(nativeAdViewBinder.admobMediaView)
                iconImageId(nativeAdViewBinder.iconImageViewId)
                titleId(nativeAdViewBinder.titleTextViewId)
                textId(nativeAdViewBinder.textTextViewId)
                callToActionId(nativeAdViewBinder.ctaButtonId)
                privacyInformationIconImageId(nativeAdViewBinder.privacyInformationIcon)
            }.build().let { registerAdRenderer(GooglePlayServicesAdRenderer(it)) }

            FacebookAdRenderer.FacebookViewBinder.Builder(nativeAdViewBinder.layout).apply {
                mediaViewId(nativeAdViewBinder.facebookMediaView)
                adIconViewId(nativeAdViewBinder.iconImageViewId)
                adChoicesRelativeLayoutId(nativeAdViewBinder.facebookAdChoicesRelativeLayout)
                advertiserNameId(nativeAdViewBinder.facebookAdvertiserNameTextView)
                titleId(nativeAdViewBinder.titleTextViewId)
                textId(nativeAdViewBinder.textTextViewId)
                callToActionId(nativeAdViewBinder.ctaButtonId)
            }.build().let { registerAdRenderer(FacebookAdRenderer(it)) }
        }.makeRequest()
    }

    override fun takeAdView(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder): View? {
        val nativeAd = nativeAdMap[adUnitId]?.apply {
            setMoPubNativeEventListener(object : NativeAd.MoPubNativeEventListener {
                override fun onClick(view: View?) {
                    onClicked(adUnitId)
                }

                override fun onImpression(view: View?) {
                    onImpressed(adUnitId)
                }
            })
        } ?: return null

        nativeAdMap.remove(adUnitId)
        val viewBinder = ViewBinder.Builder(0).build()
        val converterView = null
        val parent = null
        return AdapterHelper(context, 0, 3).getAdView(
                converterView,
                parent,
                nativeAd,
                viewBinder
        )
    }

    override fun isLoading(adUnitId: String): Boolean {
        return isLoadingMap[adUnitId] ?: false
    }

    private fun onLoaded(adUnitId: String) {
        listener.forEach {
            it.onLoaded(adUnitId)
        }
    }

    private fun onFailedToLoad(adUnitId: String) {
        listener.forEach {
            it.onFailedToLoad(adUnitId)
        }
    }

    private fun onImpressed(adUnitId: String) {
        listener.forEach { it.onImpressed(adUnitId) }
    }

    private fun onClicked(adUnitId: String) {
        listener.forEach { it.onClicked(adUnitId) }
    }
}